/* ============================================================================
 * File Name: DDS32_01 v0_0 (build_02)
 * Development version of the DDS32 component
 * 
 * Description:
 *   DDS32: up to 32-bit DDS frequency generator component (v0.0)
 *   adjustable resolution: 8, 16, 24, 32-bits
 *   realized in UDB datapath
 *
 * Credits:
 *   based on original DDS code by <voha6>:
 *   http://kazus.ru/forums/showthread.php?t=21022&page=10
 *   05.01.2011
 * 
 *   Special thanks to <pavloven>, <kabron>
 *
 * Note:
 *   frequency resolution = clock_frequency / 2^N, N=8, 16, 24, 32 
 *   frequency maximum    = clock_frequency / 2
 *   frequency minimum    = clock_frequency / 2^N
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/



#include <project.h>


static volatile CYBIT flag_Timer = 0 ; //semaphore   
CY_ISR(isr_Timer) // ISR Timer to update frequency at regular interval
{
    flag_Timer = 1;
}

//===========================================
// Function prototypes
//===========================================
void UpdateDDSfrequency();



//===========================================
// Global variables
//===========================================

    double FStart = 10000.0f;      // start frequency
    double FStep  = 1000.0f;        // frequency increment step  
    uint16 NSteps = 255u;           //number of increment steps (FStop=FStart + FStep * NSteps); Npoints = NSteps + 1   
    

void Initialize()
{   
    CyGlobalIntEnable;              //enable global interrupts.
    isr_Timer_StartEx(isr_Timer);   //start DDS update timer

    DDS_Start();                    // initialise and enable
}


int main()
{
    Initialize();
 

    //for(;;){} //test    
    
    for(;;) //forever
    {
         
        // update frequency on interrupt 
        if (flag_Timer) 
    	{    
            flag_Timer = 0;         
            UpdateDDSfrequency();    
        }
           
    } 
    
    
} // main()



//==============================================================================
// Upload next frequency value into DDS buffer
// for the next hardware load pulse
//==============================================================================

void UpdateDDSfrequency()
{
    static int16 counter = -1;  //use static to keep  recorded value
    static int16 increment = 0; //use static to keep  recorded value
    double Freq;                //
    
     
    //move phase back-forward between 0 and NSteps 
    if (counter >= NSteps) increment = -1; else //go backwards
    if (counter <= 0) increment = 1;            //go forward
        
    counter += increment;
    Freq = FStart + FStep * counter;
    
    DDS_SetFrequency(Freq);   //Upload next frequency value into DDS buffer
        
}


